# ruff: noqa: UP006 UP007
import typing as ta

from omlish.lite.typing import Func1

from .apps import DeployAppManager
from .paths.manager import DeployPathsManager
from .specs import DeploySpec
from .tags import DeployAppRev
from .tags import DeployTagMap
from .tags import DeployTime
from .types import DeployHome


class DeployDriverFactory(Func1[DeploySpec, ta.ContextManager['DeployDriver']]):
    pass


class DeployDriver:
    def __init__(
            self,
            *,
            spec: DeploySpec,
            home: DeployHome,
            time: DeployTime,

            paths: DeployPathsManager,
            apps: DeployAppManager,
    ) -> None:
        super().__init__()

        self._spec = spec
        self._home = home
        self._time = time

        self._paths = paths
        self._apps = apps

    async def drive_deploy(self) -> None:
        self._paths.validate_deploy_paths()

        #

        deploy_tags = DeployTagMap(
            self._time,
            self._spec.key(),
        )

        #

        for app in self._spec.apps:
            app_tags = deploy_tags.add(
                app.app,
                app.key(),
                DeployAppRev(app.git.rev),
            )

            await self._apps.prepare_app(
                app,
                self._home,
                app_tags,
            )



# ruff: noqa: UP006 UP007
import datetime
import typing as ta

from omlish.lite.typing import Func0

from .tags import DeployTime


DEPLOY_TAG_DATETIME_FMT = '%Y%m%dT%H%M%SZ'


DeployManagerUtcClock = ta.NewType('DeployManagerUtcClock', Func0[datetime.datetime])


class DeployManager:
    def __init__(
            self,
            *,

            utc_clock: ta.Optional[DeployManagerUtcClock] = None,
    ):
        super().__init__()

        self._utc_clock = utc_clock

    def _utc_now(self) -> datetime.datetime:
        if self._utc_clock is not None:
            return self._utc_clock()  # noqa
        else:
            return datetime.datetime.now(tz=datetime.timezone.utc)  # noqa

    def make_deploy_time(self) -> DeployTime:
        return DeployTime(self._utc_now().strftime(DEPLOY_TAG_DATETIME_FMT))
